module  pd_54to270 (
	//inputs
	vco,
	refclk,
	reset,
	vcoisfast,
	vcoisslow,
	vco_tc,
	refclk_tc
	);

///////////////////////////////////////////////////
// port declarations
input	vco;
input	refclk;
input	reset;

output	vcoisfast;
output	vcoisslow;
output	vco_tc ;
output	refclk_tc;

///////////////////////////////////////////////////
wire	vcoisfast;
wire	vcoisslow;
wire	pd_rst;



////////////////////////////////////////////////////////
//divide counter,  
//divide vco (270MHz) by 270 to get 1MHz the comparison 
//frequency.
//512 - 243(0xF3; 10'b00_1111_0011) + 1 = 270 .
////////////////////////////////////////////////////////
reg	[9:2]	vcocount;	//10bits - 2bits = 8bits
reg	[3:0]	vcocntlow;
wire		vco_tc       /* synthesis syn_keep = 1 */ ;
wire		vcocntlow_tc /* synthesis syn_keep = 1 */ ;

assign vco_tc = vcocount[9] | reset ;
assign vcocntlow_tc = vcocntlow[3] ;
////////////////////////////////////////////////////////

initial
	begin
	vcocount 	= 0 ;
	vcocntlow	= 0 ;
	end

always @ (posedge vco ) begin 
	if (vco_tc) begin
		vcocount 	<= #1 8'b00_1111_00 ;
	end
	else if( vcocntlow_tc ) begin
		vcocount	<= #1 vcocount + 1;
	end
end

////////////////////////////////////////////////////////
//lower 2 bits of the vco counter.
//shift reg counter is one hot code;
//bit 0 is 2'b00
//bit 1 is 2'b01
//bit 2 is 2'b10
//bit 3 is 2'b11
always @ (posedge vco ) begin 
	if (vco_tc) begin
		vcocntlow	<= #1 4'b1000;	// init to 3
	end
	else begin
		vcocntlow	<= #1 {vcocntlow[2:0], vcocntlow[3]};
	end
end

//////////////////////////////////////////////////////////////
reg	[6:2]	refclkcount;
reg	[3:0]	refclkcntlow ;
wire		refclk_tc /* synthesis syn_keep = 1 */ ;
wire		refclkcntlow_tc /* synthesis syn_keep = 1 */ ;

assign refclk_tc = refclkcount[6] | reset;
assign refclkcntlow_tc = refclkcntlow[3];


initial
	begin
	refclkcount 	= 0 ;
	refclkcntlow	= 0 ;
	end

/////////////////////////////////////////////////////////////
//ref clock divider
//divide the ref clock (54MHz) by 54 to get compare freq of 1MHz
//64 - 11(0x0B; 7'b000_1011) + 1 = 54 
always @ (posedge refclk ) begin 
	if (refclk_tc) begin 
		refclkcount 	<= #1 5'b000_10;
	end
	else if ( refclkcntlow_tc ) begin
		refclkcount	<= #1 refclkcount + 1;
	end
end

///////////////////////////////////////////
//lower 2 bits of the ref counter.
//shift reg counter is one hot code;
//bit 0 is 2'b00
//bit 1 is 2'b01
//bit 2 is 2'b10
//bit 3 is 2'b11
always @ (posedge refclk ) begin 
	if (refclk_tc) begin
		refclkcntlow	<= #1 4'b1000;  //the "01" part of the 0x0B
	end
	else begin
		refclkcntlow	<= #1 {refclkcntlow[2:0], refclkcntlow[3]};
	end
end

/////////////////////////////////////////////////////////
//phase detect for reference
//////////////////////////////////////////////
// set on the rising edge of the vco /ref
//pdetfast is active when the vco is faster than the ref xtal
//pdetslow is active when the vco is slower than the ref xtal
FDC pdfastreg     ( .D(1'b1), .C(vco_tc),    .CLR(pd_rst), .Q(vcoisfast));
FDC pdslowreg     ( .D(1'b1), .C(refclk_tc), .CLR(pd_rst), .Q(vcoisslow));
LUT2 pdrst_and2_0 (.I0(vcoisfast), .I1(vcoisslow), .O(pd_rst));
// synthesis attribute init of pdrst_and2_0 is 4'h8 ;
//defparam pdrst_and2_0.INIT = 4'h8;  //pdetfast & pdetslow



endmodule
