///////////////////////////////////////////////////////////////////////////////////
//
// PLL Phase detector for XAPP250 using Maxim VCO for CDR
//
//
//
//                  Author: Leonard Dieguez
//                  System IO specialist Field Applications Engineer
//
//                  Video Applications
//                  Xilinx, Inc.
//
//                  Copyright (c) 2003 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Oct. 10, 2003
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision: Ver. 0.1
//															
// Other modules instanced in this design:
//
//------------------------------------------------------------------------------
//Description of module:
//
// This module instances modules that through control of a VCO / VCXO will preform 
//frequency synthesis and clock and data recovery (CDR).
//
// This is provided for example purposes and has be verified in hardware running 
// at 270Mbps using SDI and ASI video streams. See application note XAPP250 for 
// more information on the style CDR using the Maxim 270Mhz VCO.
/////////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 1 ps

module  phasedet_v01 (
	// inputs
	vco,
	refclk,
	reset,
	data,
	pd_select,
	//outputs
	dataout,
	mkvcofaster,
	mkvcoslower,
	vco_tc,
	refclk_tc
);
	
///////////////////////////////////////////////////////////////
// port declarations
input		vco;
input		refclk;
input		reset;
input		data;
input		pd_select;

// outputs
output	[1:0]	dataout;

output		mkvcofaster;
output		mkvcoslower;
output		vco_tc;
output		refclk_tc;



//////////////////////////////////////////////////////////
// This module does frequency synthesis
// what is needed is a reference clock input and the VCO/VCXO inputs
// This example uses a 270MHz VCO and will synthesis the correct frequency from
// the reference clock. Normal reference frequencies are for example 27 MHz or 54MHz. 
// the comparison frequency is 1MHz to keep phase noise low. Lower comparison frequencies 
// can be used. The user may want to experiment with lower comparison frequencies.
pd_27to270 pdvco2ref(
	//inputs
	.vco(vco),
	.refclk(refclk),
	.reset(reset),
	//outputs
	.vcoisfast(vcoisfast),
	.vcoisslow(vcoisslow),
	.vco_tc(vco_tc),
	.refclk_tc(refclk_tc)
);



///////////////////////////////////////////////////////////////////
// CDR module. this controls the VCO or VCXO to recover the clock. 
// currently does not have locked detector information. 
pd_clk2data_v22 pd_clk2data(
	//inputs
	.clk(~vco),	//run the module on the falling edge of the VCO
	.reset(reset),
	.data(data),	// data sampled with respect to the rising edge of the
			// clock
	//outputs
	.data_out(dataout),
	.mkclkfaster(mkclkfaster1),
	.mkclkslower(mkclkslower1)
	);


/////////////////////////////////////////////////////////////////
// currently this is controlled with an external switch
// the user will want to control with a locked detector. 
// first lock to the reference clock after locked then switch to the 
// CDR PLL in "pd_clk2data"
/////////////////////////////////////////////////////////////////
MUXF5 mvfast (.I0(vcoisslow), .I1(mkclkfaster1), .S(pd_select), .O(mkvcofaster));
MUXF5 mvslow (.I0(vcoisfast), .I1(mkclkslower1), .S(pd_select), .O(mkvcoslower));



endmodule